////////////////////////////////////////////////////////////////////////////////
//
// Fracter - A simple Mandelbrot Set viewer.
//
// Copyright (C) 2010 Sarah Eris Horsley Caffee
//
// This file is part of Fracter.
//
// Fracter is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
// App.h
// Application class definition
//
////////////////////////////////////////////////////////////////////////////////

#ifndef APP_H
#define APP_H

#include <vector>

#include "SDL.h"

#include "EventHandler.h"
#include "Fractal.h"

class App : public EventHandler
   {
   public:

   App();
   ~App();

   int run();


   private:
   // Disallow copy constructor and operator=
   App(const App &);
   App & operator=(const App &);


   // main loop functions
   bool init();
   void update();
   void render();
   void cleanup();


   // Overrides of EventHandler:
   void on_InputFocus();
   void on_InputBlur();
   void on_KeyDown(SDL_KeyboardEvent key);
   void on_KeyUp(SDL_KeyboardEvent key);
   void on_MouseMove(SDL_MouseMotionEvent motion);
   void on_LButtonDown(SDL_MouseButtonEvent button);
   void on_LButtonUp(SDL_MouseButtonEvent button);
   void on_Expose();
   void on_Quit();


   // utility functions
   int mouse_move_width(int mx, int my);
   bool create_new_surface(SDL_Surface * &surface);
   void set_julia_k(int x, int y);
   void restore_view();
   void save_view();
   void set_caption();
   bool save_image();

   static Uint32 timer_callback(Uint32 interval, void *param);

   int get_next_file_num();

#if !defined(WIN32)
   static int scandir_filter(const struct dirent * d);
#endif


   // data members
   bool running;
   bool redraw;

   SDL_Surface * surf_display;
   SDL_Surface * surf_selection;
   SDL_Surface * surf_fractal;

   int selection_x, selection_y, selection_width;
   bool setting_zoom;
   bool can_set_julia_k;
   bool setting_julia_k;

   static std::vector<View *> old_views;

   Fractal * fractal;
   };

#endif
